<template>
  <div class="home">
    <!-- 搜索 -->
    <map-search
      ref="mapSearch"
      @select="onSelect"
      @search="onMapSearch"
      @clickEmployee="onClickEmployee"
      @clickStation="toStation"
      @clickPoi="toPoi"
      @clear="onSearchClear"
      @focusChange="onFocusChange"
      @railChange="railChange"
    ></map-search>
    <!-- 警情上报按钮 -->
    <div v-if="!onSearchMode" class="police-report-btn" @click="openPoliceReport"></div>
    <!-- 警情上报弹框 -->
    <police-report
      :visible.sync="showPoliceReport"
      :tabs="levelCountTabList"
      :data.sync="levelCounts"
      @dispatch="policeReportDispatch"
      @clickEmployee="policeReportEmployee"
    ></police-report>
    <!-- 工具栏 -->
    <tool-bar v-if="!onSearchMode" @changeLayer="changeLayer" @MsgListVisible="MsgListVisible"></tool-bar>
    <!-- 地图 -->
    <a-map
      ref="map"
      class="map"
      :load-options="loadOptions"
      :map-options="mapOptions"
      :layers="layers"
      :markers="markers"
      :move-marker="moveMarker"
      :move-along="moveAlong"
      :speed-ratio="speedRatio"
      @complete="onMapComplete"
      @clickMarker="onClickMarker"
      @move="onMove"
    ></a-map>
    <!-- 弹框 -->
    <info-panel :visible.sync="showInfo" @close="onPanelClose">
      <!-- 保安员 -->
      <info-employee
        ref="infoEmployee"
        v-show="type === 'employee'"
        :data="employee"
        @character="onClickCharacter"
        @callVideo="dispatchOneVideo"
        @videoLocalMute="videoLocalMute"
        @callAudio="dispatchOneAudio"
        @toCompany="toCompany"
        @toStation="toStation"
        @dispatch="onSearchAround"
        @track="openTrack"
      ></info-employee>
      <!-- 企业 -->
      <info-company v-show="type === 'company'" :data="company" @return="returnPreInfo"></info-company>
      <!-- 驻勤点 -->
      <info-station
        v-show="type === 'station'"
        :data="station"
        @toCompany="toCompany"
        @clickStationEmployees="clickStationEmployees"
        @dispatch="onSearchAround"
        @railList="showRailList"
      ></info-station>
      <info-rail-list
        v-show="type === 'railList'"
        :data="railList"
        :station="station"
        @showSecurityMan="showSecurityManFromRailList"
        @showDetail="showRailDetail"
        @backToList="backToList"
      ></info-rail-list>
      <info-security-man v-show="type === 'securityManList'" :data="securityManList" :station="station"></info-security-man>
      <!-- 地点 -->
      <info-location v-show="type === 'location'" :data="location" @searchAround="onSearchAround"></info-location>
      <!-- 调度弹框 -->
      <info-dispatch
        v-show="type === 'dispatch'"
        :data="dispatch"
        @dispatch="onDispatch"
        @conversation="createGroupConversation"
      ></info-dispatch>
      <!-- 历史轨迹 -->
      <info-track
        ref="infoTrack"
        v-show="type === 'track'"
        :data="track"
        @setTrack="onSetTrack"
        @refresh="onRefresh"
        @startMove="onStartMove"
        @stopMove="onStopMove"
        @pauseMove="onPauseMove"
        @backStep="onBackStep"
        @nextStep="onNextStep"
        @cancel="trackCancel"
        @trackList="showTrackList"
      ></info-track>
      <info-track-list v-show="type === 'trackList'" :data="trackList"></info-track-list>
    </info-panel>
    <!-- 即时通信 -->
    <current-conversation :visible.sync="showConversation"></current-conversation>
    <!-- 消息列表 -->
    <message-list :visible.sync="showMsgList" @checkoutConversation="checkoutConversation"></message-list>
    <!-- 音视频 -->
    <call-layer
      ref="callLayer"
      :member-list.sync="memberList"
      :all-members="allMembers"
      :type="callType"
      :group-call="groupCall"
      :yardman-type="yardmanType"
      :audio="stream.audio"
      :video="stream.video"
      @showMembers="showMembers"
    ></call-layer>
    <!-- 音视频成员 -->
    <calling-members
      :visible.sync="callingMembersVisible"
      :member-list.sync="memberList"
      :all-members.sync="dispatch.employeeList"
      :type="callType"
      :yardman-type="yardmanType"
      @sendVideoRoomMsg="sendVideoRoomMsg"
    ></calling-members>
  </div>
</template>

<script>
import { AMap } from "@/components/AMap";
import mapSearch from "@components/mapSearch/index.vue";
import infoPanel from "@components/infoPanel/index.vue";
import infoEmployee from "@components/infoPanel/infoEmployee.vue";
import infoCompany from "@components/infoPanel/infoCompany.vue";
import infoStation from "@components/infoPanel/infoStation.vue";
import infoLocation from "@components/infoPanel/infoLocation.vue";
import infoDispatch from "@components/infoPanel/infoDispatch.vue";
import infoTrack from "@components/infoPanel/infoTrack.vue";
import infoTrackList from "@components/infoPanel/infoTrackList.vue";
import infoRailList from "@components/infoPanel/infoRailList.vue";
import infoSecurityMan from "@components/infoPanel/infoSecurityMan.vue";
import toolBar from "@components/toolBar.vue";
import currentConversation from "@components/timApp/conversation/current-conversation.vue";
import messageList from "@components/timApp/conversation/message-list.vue";
import CallLayer from "@components/timApp/trtc/calling-index.vue";
import CallingMembers from "@components/timApp/trtc/calling-members.vue";
import policeReport from "@components/policeReport.vue";

import { getEmployeeDetail, getEmployees } from "@/api/employee-controller";
import { getCompanyDetail } from "@/api/company-controller";
import { getStationDetail } from "@/api/security-station-controller";
import { getLevelCount } from "@/api/police-incident-controller";

import { textSlice } from "@/api/common";
import tim from "@/tim";
import { mapGetters, mapState } from "vuex";
import { getCurrentUser } from "@/api/tenant-user-controller";
import { Toast } from "vant";
import _webSocket from "@/utils/websocket";

export default {
  name: "Home",
  components: {
    [Toast.name]: Toast,
    CallLayer,
    CallingMembers,
    AMap,
    mapSearch,
    infoPanel,
    infoEmployee,
    infoCompany,
    infoStation,
    infoLocation,
    infoDispatch,
    infoTrack,
    infoTrackList,
    infoRailList,
    infoSecurityMan,
    currentConversation,
    messageList,
    toolBar,
    policeReport,
  },
  data() {
    return {
      loadOptions: {
        version: "2.0",
      },
      mapOptions: {
        viewMode: "3D", //是否为3D地图模式
        pitchEnable: true,
        rotateEnable: true,
      },
      layers: [{ type: "default" }],
      showMsgList: false,
      markers: [],
      moveMarker: {
        icon: require("@/assets/images/mapSearch/在岗_保安员.png"),
        anchor: "bottom-center",
      },
      moveAlong: {
        autoRotation: false,
      },
      speedRatio: 1,
      moveStatus: "",
      info: {},
      employee: {},
      company: {},
      station: {},
      location: {},
      dispatch: {},
      track: {},
      trackList: [],
      railList: [],
      securityManList: [],
      securityManBackType: "",
      showInfo: false,
      openFrom: "",
      type: "",
      showConversation: false,
      isComplete: false,
      onSearchMode: false,
      memberList: [],
      allMembers: [],
      stream: {
        audio: true,
        video: true,
      },
      //调度类型：1视频调度 2 视频上拉：除了视频上拉传2，其他音频调度与视频调度都传1
      yardmanType: "1",
      callType: "video",
      groupCall: false,
      showPoliceReport: false,
      levelCounts: {},
      levelCountTabList: [
        {
          label: "特别重大",
          number: "0",
          name: "greatsCount",
          level: "Greats",
          bgColor: "#A80000",
        },
        {
          label: "重大",
          number: "0",
          name: "greatCount",
          level: "Great",
          bgColor: "#F48200",
        },
        {
          label: "较大",
          number: "0",
          name: "moreCount",
          level: "More",
          bgColor: "#F7BE00",
        },
        {
          label: "一般",
          number: "0",
          name: "comonCount",
          level: "Comon",
          bgColor: "#00B40C",
        },
      ],
      callingMembersVisible: false,
      ws: null,
      webSocketUrl: "",
      circleFigure: null,
      circleEditorObject: null,
      rectangleFigure: null,
      rectangleEditor: null,
      polygonFigure: null,
      polygonEditor: null,
    };
  },
  computed: {
    ...mapState({
      currentConversation: (state) => state.conversation.currentConversation,
      currentMessageList: (state) => state.conversation.currentMessageList,
      rail: (state) => state.app.rail,
      roomId: (state) => state.trtc.roomId,
    }),
    ...mapGetters(["sysUserInfo"]),
  },
  methods: {
    //ws初始化
    initWs() {
      const dispatchToken = localStorage.getItem("dispatchToken");
      if (!dispatchToken) return false;
      this.webSocketUrl = `/websocket/police_incidents?dispatchToken=${dispatchToken}`;
      this.ws = _webSocket({
        url: this.webSocketUrl,
        onMessage: this.onMessage,
        heart: {
          timer: 5000,
          message: "keep alive",
        },
        isReconnect: true,
        onClose: this.onClose,
      });
    },
    onClose() {
      this.initWs();
    },
    onMessage(evt) {
      // console.log("ws data ===", evt);
      if (evt) {
        if (evt.data.includes("keep alive")) {
          //心跳
        } else {
          const data = JSON.parse(evt.data);
          console.log("ws data ===", data);
          const { type, message } = data;
          if (type === "PoliceIncidents") {
            const content = message.contents[0];
            if (content.category === "Sos") {
              // TODO 处理SOS
            } else {
              // TODO 处理警情上报
            }
          } else if (type === "TRTC") {
            const content = message.contents[0];
            console.log(content);
            const { action, tencentUserId, roomId } = content;
            // 如果不是本人的房间直接忽略
            if (this.roomId && roomId === this.roomId) {
              if (action === "handUp" || action === "cheese") {
                // this.$message.info("有1人举手");
                Toast({
                  message: "有1人举手",
                  position: "top",
                  icon: require("@/assets/images/conversation/移动端举手.png"),
                });
              }
              this.memberList.forEach((member) => {
                if (member.tencentUserId === tencentUserId) {
                  member.action = action;
                }
              });
              console.log(this.memberList);
            }
          }
        }
      }
    },
    onSelect() {
      this.showInfo = false;
      this.markers = [];
    },
    onMapSearch({ type, data }) {
      if (type === 3) {
        console.log("onMapSearch", data);
        this.markers = data.map((poi) => {
          const coordinates = poi.location.split(",");
          return {
            position: [Number(coordinates[0]), Number(coordinates[1])],
            anchor: "bottom-center",
            icon: require("@/assets/images/map/icons/mark_b.png"),
            // label: {
            //   content: index + 1,
            //   direction: "center",
            //   offset: [0, 0],
            // },
            extData: poi,
          };
        });
        console.log("markers", this.markers);
      }
    },
    /**
     * @安保员详情
     * @param id
     * @param showMore
     */
    onClickEmployee({ id }, showMore) {
      // this.showPoliceReport = false;
      // 聚焦并弹框
      getEmployeeDetail(id)
        .then((employeeDetail) => {
          console.log("onClickEmployee", employeeDetail);
          // this.employee = employeeDetail;
          let iconUrl;
          if (employeeDetail.dutyStatusText === "在岗") {
            iconUrl = require("@/assets/images/mapSearch/在岗_保安员.png");
          } else {
            iconUrl = require("@/assets/images/mapSearch/离岗_保安员.png");
          }
          employeeDetail["nameSlice"] = employeeDetail.name.slice(-2);
          employeeDetail["idCardNoHide"] = this.parseStringToStar(employeeDetail.idCardNo);
          employeeDetail["contactHide"] = this.parseStringToStar(employeeDetail.contact);
          employeeDetail["type"] = "employee";
          employeeDetail["companyNameSlice"] = textSlice(employeeDetail.companyName); //多出部分省略号显示
          employeeDetail["securityStationNameSlice"] = textSlice(employeeDetail.securityStationName); //多出部分省略号显示

          this.markers = [
            {
              position: [Number(employeeDetail.longitude), Number(employeeDetail.latitude)],
              anchor: "bottom-center",
              icon: iconUrl,
              extData: employeeDetail,
            },
          ];
          this.$refs.map.map.setZoomAndCenter(14, [Number(employeeDetail.longitude), Number(employeeDetail.latitude)]);
          this.showInfoByRowClick(employeeDetail, "employee", showMore);
        })
        .catch((err) => {
          console.log(err);
        });
    },
    parseStringToStar(str) {
      return str.length > 2 ? str.substr(0, 2) + new Array(str.length - 2).join("*") + str.substr(-2) : str;
    },
    /**
     * @显示弹框
     * @param data
     * @param type
     * @param showMore
     */
    showInfoByRowClick(data, type, showMore) {
      console.log(`open info panel ${type}'s data ====>`, data);
      this.type = type;
      if (type === "employee") {
        this.employee = { ...{}, ...data };
        this.$refs.infoEmployee.showContent = showMore;
        this.showInfo = true;
      } else if (type === "company") {
        this.company = { ...{}, ...data };
        this.showInfo = true;
      } else if (type === "station") {
        this.station = { ...{}, ...data };
        this.showInfo = true;
      } else if (type === "location") {
        this.location = { ...{}, ...data };
        this.showInfo = true;
      } else if (type === "dispatch") {
        this.getEmployeeToDispatch(data, type);
      } else if (type === "track") {
        this.track = { ...{}, ...data };
        this.showInfo = true;
      }
    },
    async onMapComplete() {
      if (Object.keys(this.sysUserInfo).length > 0) {
        const { longitude, latitude } = await getCompanyDetail(this.sysUserInfo?.organizationId);
        this.$refs.map.setCenter([longitude, latitude]);
      } else {
        const { organizationId } = await getCurrentUser();
        if (organizationId) {
          const { longitude, latitude } = await getCompanyDetail(organizationId);
          this.$refs.map.setCenter([longitude, latitude]);
        }
      }
    },
    onClickMarker(e) {
      const { target } = e;
      this.info = { ...target.getExtData() };
      this.showInfo = true;
    },
    onClickCharacter() {
      console.log(this.employee);
      tim.getConversationProfile(`C2C${this.employee.tenantImUserId}`).then(({ data }) => {
        console.log("getConversationProfile---", data);
        this.$store.commit("updateCurrentConversation", {
          ...this.employee,
          ...data.conversation,
        });
        const { conversationID } = data.conversation;
        this.$store.dispatch("getMessageList", conversationID);
        console.log(this.currentConversation);
        this.isComplete = false;
        this.showConversation = true;
      });
    },
    startVideoDispatch(groupCall) {
      // this.$bus.$emit("video-call");
      this.initCalling(groupCall).then(() => {
        // this.$refs.callLayer.showCalling = true;
        this.$bus.$emit("video-call");
      });
    },
    initCalling(groupCall) {
      return new Promise((resolve) => {
        this.groupCall = groupCall;
        resolve();
      });
    },
    /**
     * @企业详情
     * @param info
     * @param infoType
     */
    toCompany(info, infoType) {
      getCompanyDetail(info.organizationId)
        .then((company) => {
          console.log("company", company);
          company["returnWin"] = infoType;
          company["returnDetail"] = info;
          company["type"] = "company";
          company["registerAddressSlice"] = textSlice(company.registerAddress);
          this.markers = [
            {
              position: [Number(company.longitude), Number(company.latitude)],
              anchor: "bottom-center",
              icon: require("@/assets/images/mapSearch/公司.png"),
              label: {
                content: 0,
                direction: "center",
                offset: [0, 0],
              },
              extData: company,
            },
          ];
          this.$refs.map.map.setZoomAndCenter(14, [Number(company.longitude), Number(company.latitude)]);
          this.showInfoByRowClick(company, "company");
        })
        .catch((err) => {
          console.log(err);
        });
    },
    /**
     * @驻勤点详情
     * @param info
     * @param infoType
     */
    toStation(info, infoType) {
      getStationDetail(info.securityStationId)
        .then((station) => {
          console.log("station", station);
          station["returnWin"] = infoType;
          station["returnDetail"] = info;
          this.onClickStation(station);
        })
        .catch((err) => {
          console.log(err);
        });
    },
    toPoi(poi) {
      let tempMarker;
      this.markers.forEach((marker) => {
        if (marker.extData.id === poi.id) {
          marker.icon = require("@/assets/images/map/icons/mark_b.png");
          this.showInfoByRowClick(poi, "location");
          tempMarker = marker;
        }
      });
      this.markers = [];
      console.log(tempMarker);
      this.markers.push(tempMarker);
      this.$refs.map.setZoomAndCenter(14, tempMarker.position);
    },
    // 搜驻勤点后触发
    onClickStation(station, type) {
      // 聚焦并弹框
      console.log("onClickStation", station);
      if (type === "employee") {
        this.onClickEmployee(this.employee);
      } else {
        station["type"] = "station";
        station["companyNameSlice"] = textSlice(station.companyName); //多出部分省略号显示
        station["securityStationNameSlice"] = textSlice(station.name); //多出部分省略号显示
        station["superviseDepartNameSlice"] = textSlice(station.superviseDepartName); //多出部分省略号显示
        station["addressSlice"] = textSlice(station.address); //多出部分省略号显示
        console.log("station:", station);
        // this.station = { ...{}, ...station };
        this.markers = [
          {
            position: [Number(station.longitude), Number(station.latitude)],
            anchor: "bottom-center",
            // icon: require("@/assets/images/mapSearch/驻勤点.png"),
            icon: {
              image: station.stationTypes[0].mapIcon,
              imageSize: [56, 56],
            },
            extData: station,
          },
        ];
        this.$refs.map.map.setZoomAndCenter(14, [Number(station.longitude), Number(station.latitude)]);
        this.showInfoByRowClick(station, "station");
      }
    },
    /**
     * @点击驻勤点人员,修改成驻勤点人员撒点
     * @getEmployees接口没有返回经纬度,先替换成驻勤点人员列表
     */
    clickStationEmployees({ id }) {
      this.securityManBackType = "station";
      this.showSecurityMan(id);
    },
    onSearchClear() {
      this.markers = [];
      this.showInfo = false;
    },
    onFocusChange(value) {
      this.onSearchMode = value;
    },
    onPanelClose() {
      // 如果从调度弹框关闭，自动弹出上一个弹框
      console.log("onPanelClose", this.type);
      if (this.type === "dispatch") {
        this.allMembers = [];
        this.$refs.mapSearch.back(true);
        switch (this.openFrom) {
          case "employee":
            this.showInfoByRowClick(this.employee, "employee");
            break;
          case "station":
            this.showInfoByRowClick(this.station, "station");
            break;
          case "location":
            this.showInfoByRowClick(this.location, "location");
            break;
          case "policeReport":
            this.showPoliceReport = true;
            break;
        }
      } else if (this.type === "track") {
        // 关闭轨迹弹框返回保安员弹框
        if (this.openFrom === "employee") {
          this.showInfoByRowClick(this.employee, "employee");
        }
      } else if (this.type === "trackList") {
        this.showInfoByRowClick(this.track, "track");
      } else if (this.type === "railList") {
        this.backToList(this.station);
        this.showInfoByRowClick(this.station, "station");
      } else if (this.type === "securityManList") {
        // 有可能从电子围栏返回 也有可能从驻勤点返回
        this.type = this.securityManBackType;
        this.showInfo = true;
      } else if (this.type === "employee") {
        this.allMembers = [];
      }
    },
    /**
     * @返回上一个弹框
     * @param data 上一个弹框的信息
     * @param type 上一个弹框是哪种弹框
     */
    returnPreInfo(data, type) {
      console.log(type);
      console.log(data);
      if (type === "employee") {
        this.onClickEmployee(data);
      } else if (type === "station") {
        data.securityStationId = data.id;
        this.toStation(data);
      }
    },
    changeLayer(satellite) {
      if (satellite) {
        // 卫星图
        this.layers = [{ type: "Satellite" }, { type: "RoadNet" }];
      } else {
        this.layers = [{ type: "default" }];
      }
    },
    MsgListVisible(visible) {
      this.showMsgList = visible;
    },
    /**
     * @周边查找
     * @param data
     * @确保data参数内有必要字段: longitude & latitude
     */
    onSearchAround(data) {
      this.openFrom = data.from;
      this.type = "dispatch";
      this.$refs.mapSearch.dispatch();
      this.showInfoByRowClick(data, "dispatch");
    },
    openTrack(data) {
      this.openFrom = "employee";
      this.showInfoByRowClick(data, "track");
    },
    railChange(distance) {
      this.$store
        .dispatch("setRail", {
          distance,
        })
        .then(() => {
          getEmployees({
            pageIndex: 1,
            pageSize: 1000,
            // securityStationId: this.rail.id,
            // longitude: this.rail.longitude,
            // latitude: this.rail.latitude,
            // distance: this.value,
            searchModel: "Geo",
            deskTypes: ["Normal", "Proprietor"],
            ...this.rail,
          })
            .then((stationEmployees) => {
              this.setEmployee(stationEmployees);
            })
            .catch((err) => {
              console.log(err);
            });
        });
    },
    //周边调度查询保安员并撒点
    getEmployeeToDispatch(data, type) {
      // if (this.value === 0) {
      //   this.$message({
      //     message: "请选择范围",
      //     type: "warning",
      //   });
      // }
      const { longitude, latitude } = data;
      this.$store
        .dispatch("setRail", {
          longitude,
          latitude,
        })
        .then(() => {
          getEmployees({
            pageIndex: 1,
            pageSize: 1000,
            searchModel: "Geo",
            deskTypes: ["Normal", "Proprietor"],
            ...this.rail,
          })
            .then((stationEmployees) => {
              this.setEmployee(stationEmployees, type, data);
            })
            .catch((err) => {
              console.log(err);
            });
        });
    },
    setEmployee(stationEmployees, type, data) {
      console.log("stationEmployees:", stationEmployees);
      this.markers = [];
      stationEmployees.forEach((item) => {
        item.active = false;
        item["nameSlice"] = item.name.slice(-2);
        item["idCardNoHide"] = this.parseStringToStar(item.idCardNo);
        item["idVisible"] = false;
        item["contactHide"] = this.parseStringToStar(item.contact);
        item["concatVisible"] = false;
        item["companyNameSlice"] = textSlice(item.companyName);
        item["securityStationNameSlice"] = textSlice(item.securityStationName);
        item["type"] = "employee";
        item["isPhone"] = false;
        item["isIdCardNo"] = false;
        item["checked"] = false;
        let iconUrl;
        if (item.dutyStatusText === "在岗") {
          iconUrl = require("@/assets/images/mapSearch/在岗_保安员.png");
        } else {
          iconUrl = require("@/assets/images/mapSearch/离岗_保安员.png");
        }
        this.markers.push({
          position: [Number(item.longitude), Number(item.latitude)],
          anchor: "bottom-center",
          icon: iconUrl,
          extData: item,
        });
      });
      this.dispatch = {
        ...{
          type,
          employeeList: stationEmployees,
        },
        ...data,
      };
      this.showInfo = true;
    },
    async openPoliceReport() {
      this.$store.commit("SET_APP_LOADING", true);
      this.levelCounts = await getLevelCount(); //获取警情上报等级数量
      this.levelCountTabList.forEach((item) => {
        item.number = this.levelCounts[item.name];
      });
      this.$store.commit("SET_APP_LOADING", false);
      this.showPoliceReport = true;
    },
    policeReportDispatch(rail) {
      console.log("===========", rail);
      rail["type"] = "rail";
      this.openFrom = "policeReport";
      this.dispatch = rail;
      this.markers = [
        {
          position: [Number(rail.longitude), Number(rail.latitude)],
          anchor: "bottom-center",
          extData: rail,
          icon: {
            image: require("@/assets/images/mapSearch/警情打点.png"),
            imageSize: [56, 56],
          },
        },
      ];
      this.showPoliceReport = false;
      this.showInfoByRowClick(this.dispatch, "dispatch");
    },
    policeReportEmployee(data) {
      this.onClickEmployee(data, true);
    },
    onSetTrack(path) {
      console.log("onSetTrack", path);
      this.$refs.map.setMoveAnimation(path);
      // this.$refs.map.moveStart()
    },
    onMove(e) {
      const { index } = e;
      this.$refs.infoTrack.setCurrentRow(index);
    },
    onRefresh(speedRatio) {
      this.$refs.map.stopMove();
      this.$refs.map.moveStart(speedRatio);
    },
    onStartMove(speedRatio) {
      this.speedRatio = speedRatio;
      if (this.moveStatus === "pause") {
        this.$refs.map.resumeMove();
      } else {
        this.$refs.map.moveStart();
      }
      this.moveStatus = "moving";
    },
    onPauseMove() {
      this.$refs.map.pauseMove();
      this.moveStatus = "pause";
    },
    onStopMove() {
      this.$refs.map.stopMove();
      this.moveStatus = "stop";
    },
    onBackStep() {
      this.$refs.map.backStep();
    },
    onNextStep() {
      this.$refs.map.nextStep();
    },
    trackCancel() {
      // this.onHideAllLayers();
      // this.infoWindow.employee.position = null;
      this.$refs.map.stopMove();
      this.$refs.map.map.clearMap();
    },
    showTrackList(list) {
      this.trackList = list;
      this.type = "trackList";
      console.log("trackList", this.trackList);
    },
    showRailList(list) {
      this.railList = list;
      this.type = "railList";
    },
    showSecurityManFromRailList(id) {
      this.securityManBackType = "railList";
      this.showSecurityMan(id);
    },
    backToList(station) {
      this.markers = [
        {
          position: [Number(station.longitude), Number(station.latitude)],
          anchor: "bottom-center",
          // icon: require("@/assets/images/mapSearch/驻勤点.png"),
          icon: {
            image: station.stationTypes[0].mapIcon,
            imageSize: [56, 56],
          },
          extData: station,
        },
      ];
    },
    showRailDetail(data, station) {
      console.log("show rail detail ", data);
      this.showMapEditor(data, station);
    },
    showMapEditor({ shapeType, shapedLocation }, station) {
      switch (shapeType) {
        case "Circle": // 圆
          this.showCircleMarker(shapedLocation, station);
          break;
        case "Polygon":
          this.showPolygonMarker(shapedLocation, station);
          break;
        case "Crib":
          this.showCribMarker(shapedLocation, station);
          break;
      }
    },
    showCircleMarker(shapedLocation, station) {
      this.marker = [
        {
          position: [Number(station.longitude), Number(station.latitude)],
          anchor: "bottom-center",
          // icon: require("@/assets/images/mapSearch/驻勤点.png"),
          icon: {
            image: station.stationTypes[0].mapIcon,
            imageSize: [56, 56],
          },
          extData: station,
        },
        {
          type: "circle",
          center: [shapedLocation.geos[0].lon, shapedLocation.geos[0].lat],
          radius: shapedLocation.geos[0].radius,
          borderWeight: 3,
          strokeColor: "#E6A23C",
          // strokeOpacity: 1,
          strokeWeight: 6,
          strokeOpacity: 1,
          fillOpacity: 0.2,
          strokeStyle: "dashed",
          strokeDasharray: [10, 10],
          // 线样式还支持 'dashed'
          fillColor: "#E6A23C",
          zIndex: 50,
        },
      ];
    },
    showPolygonMarker(shapedLocation, station) {
      console.log("show polygon marker ", station);
      const path = shapedLocation.geos.map((item) => [item.lon, item.lat]);
      this.markers = [
        {
          position: [Number(station.longitude), Number(station.latitude)],
          anchor: "bottom-center",
          // icon: require("@/assets/images/mapSearch/驻勤点.png"),
          icon: {
            image: station.stationTypes[0].mapIcon,
            imageSize: [56, 56],
          },
          extData: station,
        },
        {
          type: "Polygon",
          path,
          borderWeight: 3,
          strokeColor: "#E6A23C",
          // strokeOpacity: 1,
          strokeWeight: 6,
          strokeOpacity: 1,
          fillOpacity: 0.2,
          strokeStyle: "dashed",
          strokeDasharray: [10, 10],
          // 线样式还支持 'dashed'
          fillColor: "#E6A23C",
          zIndex: 50,
        },
      ];
    },
    showCribMarker(shapedLocation, station) {
      const path = shapedLocation.geos.map((item) => [item.lon, item.lat]);
      this.markers = [
        {
          position: [Number(station.longitude), Number(station.latitude)],
          anchor: "bottom-center",
          // icon: require("@/assets/images/mapSearch/驻勤点.png"),
          icon: {
            image: station.stationTypes[0].mapIcon,
            imageSize: [56, 56],
          },
          extData: station,
        },
        {
          type: "Polygon",
          path,
          borderWeight: 3,
          strokeColor: "#E6A23C",
          // strokeOpacity: 1,
          strokeWeight: 6,
          strokeOpacity: 1,
          fillOpacity: 0.2,
          strokeStyle: "dashed",
          strokeDasharray: [10, 10],
          // 线样式还支持 'dashed'
          fillColor: "#E6A23C",
          zIndex: 50,
        },
      ];
    },
    showSecurityMan(id) {
      getEmployees({
        pageIndex: 1,
        pageSize: 5000,
        securityStationIds: [...id],
        searchModel: "SecurityStation",
        deskTypes: ["Normal", "Proprietor"],
      }).then((res) => {
        console.log(res);
        if (res && res.length > 0) {
          this.securityManList = res.map((x) => {
            return {
              ...x,
              concatVisible: false,
              contactHide: this.parseStringToStar(x.contact),
            };
          });
          this.type = "securityManList";
        }
      });
    },
    checkoutConversation() {
      this.showMsgList = false;
      this.showConversation = true;
    },
    /**
     * @调度前参数设置
     * @param type
     * @returns {Promise<void>}
     */
    setCallProps(type) {
      if (type === "video") {
        this.stream.video = true;
        this.stream.audio = true;
        this.callType = "video";
        this.yardmanType = "1";
      } else if (type === "audio") {
        this.stream.video = false;
        this.stream.audio = true;
        this.callType = "audio";
        this.yardmanType = "1";
      } else if (type === "videoLocalMute") {
        this.stream.video = false;
        this.stream.audio = true;
        this.callType = "video";
        this.yardmanType = "2";
      }
      return Promise.resolve();
    },
    /**
     * @多人调度
     * @param type
     * @param memberList
     */
    onDispatch({ type, memberList }) {
      this.memberList = [...memberList];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      if (this.memberList.length > 0) {
        this.setCallProps(type).then(() => {
          this.startVideoDispatch(true);
        });
      } else {
        Toast.fail("至少选择一名被调度人");
      }
    },
    videoLocalMuteCall() {
      this.memberList = [this.employee];
      this.allMembers = [];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
        // member.id = member.employeeId
      });
      this.stream.video = false;
      this.stream.audio = true;
      this.yardmanType = "2";
      this.callType = "video";
      return Promise.resolve();
    },
    videoLocalMute() {
      console.log("上拉", this.info);
      this.videoLocalMuteCall().then(() => {
        this.$bus.$emit("video-call");
      });
    },
    /**
     * @视频调度1v1
     */
    dispatchOneVideo() {
      this.memberList = [this.employee];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      this.setCallProps("video").then(() => {
        this.startVideoDispatch(false);
      });
    },
    /**
     * @语音调度1v1
     */
    dispatchOneAudio() {
      this.memberList = [this.employee];
      this.memberList.forEach((member) => {
        member.state = "CONNECTING";
        member.action = "";
      });
      this.setCallProps("audio").then(() => {
        this.startVideoDispatch(false);
      });
    },
    /**
     * @设置群聊创建条件
     * @param members
     * @returns {{memberList: *}}
     */
    getOptions(members) {
      return {
        type: this.TIM.TYPES.GRP_WORK,
        name: "指挥调度",
        memberList: members.map(({ tenantImUserId }) => {
          return { userID: tenantImUserId + "" };
        }),
      };
    },
    /**
     * @创建IM文字群聊
     * @param members
     */
    createGroupConversation(members) {
      console.log(members);
      const groupOptions = this.getOptions(members);
      tim
        .createGroup({ ...groupOptions })
        .then((imResponse) => {
          Toast.success(`群组：【${imResponse.data.group.name}】创建成功`);
          this.showGroupConversation(imResponse.data);
        })
        .catch((error) => {
          Toast.fail(error.message);
        });
    },
    /**
     * @打开群聊弹框
     * @param group
     */
    showGroupConversation({ group }) {
      console.log("group data ===", group);
      tim.getConversationProfile(`GROUP${group.groupID}`).then(({ data }) => {
        console.log(data);
        this.$store.commit("updateCurrentConversation", data.conversation);
        const { conversationID } = data.conversation;
        this.$store.dispatch("getMessageList", conversationID);
        console.log(this.currentConversation);
        this.showConversation = true;
      });
    },
    showMembers() {
      // 可选择的人就是调度弹框内的人，checked的人是已经在调度的人，active的人就是要加入调度的人
      console.log(this.dispatch.employeeList);
      this.callingMembersVisible = true;
    },
    sendVideoRoomMsg(roomId, memberList, UseList) {
      this.callingMembersVisible = false;
      this.$refs.callLayer.sendRoomMessage(roomId, memberList, UseList);
    },
  },
  created() {
    this.initWs();
  },
};
</script>

<style scoped lang="less">
.home {
  width: 100%;
  height: 100%;
  position: relative;
  .map {
    z-index: 1;
  }
  .police-report-btn {
    z-index: 2;
    width: 44px;
    height: 44px;
    background: url("~@/assets/images/mapSearch/警情上报.png") no-repeat;
    background-size: 100% 100%;
    position: absolute;
    top: 44px;
    right: 12px;
  }
}
</style>
